//#pragma OPENCL EXTENSION cl_khr_fp64 : enable

#define PI (3.1415926535897932384626433832795f)
#define EPSILON (.000001f)
#define RAY_OFFSET (.0005f)
#define absf(f) (f > 0 ? f : -f)
#define OCTREE (0)
#define FOV (100)
//#define RNGTEST

bool intersect(__global int* octree, uint depth,
	float* d, float* o, float* n)
{
	
	int node;
	int level;
	int type = -1;
	
	int lx, ly, lz;
	int x, y, z;
	float tNear = INFINITY;
	float t;
	
	while (1) {
		
		// add small offset past the intersection to avoid
		// recursion to the same octree node!
		x = floor(o[0] + d[0] * RAY_OFFSET);
		y = floor(o[1] + d[1] * RAY_OFFSET);
		z = floor(o[2] + d[2] * RAY_OFFSET);
		
		node = 0;
		level = depth;
		lx = x >> level;
		ly = y >> level;
		lz = z >> level;
		
		if (lx != 0 || ly != 0 || lz != 0) {
			// outside octree!
			return false;
		}
		
		type = octree[node];
		while (type == -1) {
			level -= 1;
			lx = x >> level;
			ly = y >> level;
			lz = z >> level;
			node = octree[node + 1 + (((lx&1)<<2) | ((ly&1)<<1) | (lz&1))];
			type = octree[node];
		}

		if (type != 0) {
			return true;
		}

		t = ((lx<<level) - o[0]) / d[0];
		if (t > EPSILON) {
			tNear = t;
			n[0] = 1;
			n[1] = n[2] = 0;
		}
		t = (((lx+1)<<level) - o[0]) / d[0];
		if (t < tNear && t > EPSILON) {
			tNear = t;
			n[0] = -1;
			n[1] = n[2] = 0;
		}
		t = ((ly<<level) - o[1]) / d[1];
		if (t < tNear && t > EPSILON) {
			tNear = t;
			n[1] = 1;
			n[0] = n[2] = 0;
		}
		t = (((ly+1)<<level) - o[1]) / d[1];
		if (t < tNear && t > EPSILON) {
			tNear = t;
			n[1] = -1;
			n[0] = n[2] = 0;
		}
		t = ((lz<<level) - o[2]) / d[2];
		if (t < tNear && t > EPSILON) {
			tNear = t;
			n[2] = 1;
			n[0] = n[1] = 0;
		}
		t = (((lz+1)<<level) - o[2]) / d[2];
		if (t < tNear && t > EPSILON) {
			tNear = t;
			n[2] = -1;
			n[0] = n[1] = 0;
		}

		o[0] += d[0] * tNear;
		o[1] += d[1] * tNear;
		o[2] += d[2] * tNear;
		tNear = INFINITY;
	}
}

/* The MWC64X function is Copyright (c) 2011, David Thomas
 * Found at:
 * http://cas.ee.ic.ac.uk/people/dt10/research/rngs-gpu-mwc64x.html
 * See MWC64X.txt for the license terms affecting this function.
 */
uint MWC64X(uint2 *state)
{
    enum { A=4294883355U};
    uint x=(*state).x, c=(*state).y;
    uint res=x^c;
    uint hi=mul_hi(x,A);
    x=x*A+c;
    c=hi+(x<c);
    *state=(uint2)(x,c);
    return res;
}

float rand_float(uint2* state)
{
	return MWC64X(state) / (float)(0xFFFFFFFF);
}

void reflect(float* o, float* d, float* n)
{
	float dot = d[0]*n[0] + d[1]*n[1] + d[2]*n[2];
	d[0] -= 2 * dot * n[0];
	d[1] -= 2 * dot * n[1];
	d[2] -= 2 * dot * n[2];
	o[0] = mad(RAY_OFFSET, d[0], o[0]);
	o[1] = mad(RAY_OFFSET, d[1], o[1]);
	o[2] = mad(RAY_OFFSET, d[2], o[2]);
}

void reflect_diffuse(float* o, float* d, float* n, uint2* state)
{
	float r1 = 2 * PI * rand_float(state);
	float r2 = rand_float(state);
	float r2s = sqrt(r2);
	float uw = cos(r1) * r2s;
	float vw = sin(r1) * r2s;
	float ww = sqrt(1 - r2);
	
	float tx, ty, tz;
	float ux, uy, uz;
	float vx, vy, vz;
	
	if (fabs(n[0]) > .1f) {
		tx = 0;
		ty = 1;
		tz = 0;
	} else {
		tx = 1;
		ty = 0;
		tz = 0;
	}
	
	ux = ty * n[2] - tz * n[1];
	uy = tz * n[0] - tx * n[2];
	uz = tx * n[1] - ty * n[0];
	
	float r = 1 / sqrt(ux*ux + uy*uy + uz*uz);
	
	ux *= r;
	uy *= r;
	uz *= r;
	
	vx = uy * n[2] - uz * n[1];
	vy = uz * n[0] - ux * n[2];
	vz = ux * n[1] - uy * n[0];
	
	d[0] = ux * uw + vx * vw + n[0] * ww;
	d[1] = uy * uw + vy * vw + n[1] * ww;
	d[2] = uz * uw + vz * vw + n[2] * ww;
	r = 1 / sqrt(d[0]*d[0] + d[1]*d[1] + d[2]*d[2]);
	d[0] *= r;
	d[1] *= r;
	d[2] *= r;
	
	o[0] = mad(RAY_OFFSET, d[0], o[0]);
	o[1] = mad(RAY_OFFSET, d[1], o[1]);
	o[2] = mad(RAY_OFFSET, d[2], o[2]);
}

uint to_rgb(float* c)
{
	c[0] = max(1.f, c[0]);
	c[1] = max(1.f, c[1]);
	c[2] = max(1.f, c[2]);
	return (uint) (0xFF<<24) | ((0xFF*(uint)c[0])<<16) | ((0xFF*(uint)c[1])<<8) | (0xFF*(uint)c[2]);
}

__kernel void path_trace(__global float* output,
		uint buff_w, uint buff_h, uint nx, uint ny,
		__global float* origin,
		__global int* octree, uint depth,
		__global float* transform,
		__global uint2* seeds,
		uint num_samples)
{

	uint ix = get_global_id(0);
	uint iy = get_global_id(1);
	uint2 seed = seeds[ix + iy*nx];
	uint width;
	uint height;
	uint xoff = ix * (buff_w / nx);
	uint yoff = iy * (buff_h / ny);
	
	// calculate buffer width
	if (ix < nx-1)
		width = buff_w / nx;
	else
		width = buff_w - (buff_w / nx) * (nx-1);

	// calculate buffer height
	if (iy < ny-1)
		height = buff_h / ny;
	else
		height = buff_h - (buff_h / ny) * (ny-1);
		
	float d[3];
	float o[3];
	float n[3];
	float s[3];
	float c[3];
	
	float fov_tan = tanpi(FOV / 360.f);

	float dx, dy, dz;
	float r;
	
	s[0] = .1f;
	s[1] = 1.f;
	s[2] = -0.5f;
	r = 1 / sqrt(s[0]*s[0] + s[1]*s[1] + s[2]*s[2]);
	s[0] *= r;
	s[1] *= r;
	s[2] *= r;
	
	uint ray_depth;
	
	float sinv = 1.f / (num_samples + 1);
	bool hit;

	for (int x = 0; x < width; ++x) {
		for (int y = 0; y < height; ++y) {
			uint index = ((yoff + y) * buff_w + (xoff + x))*3;

			#ifdef RNGTEST
			output[index] = rand_float(&seed);
			output[index+1] = rand_float(&seed);
			output[index+2] = rand_float(&seed);
			#else
			dx = fov_tan * (-.5f + (float) (y + yoff) / buff_h);
			dy = -1;
			dz = fov_tan * (.5f - (float) (x + xoff) / buff_w);

			r = 1 / sqrt(dx*dx + dy*dy + dz*dz);

			dx *= r;
			dy *= r;
			dz *= r;
			
			d[0] = dx*transform[0] + dy*transform[1] + dz*transform[2];
			d[1] = dx*transform[3] + dy*transform[4] + dz*transform[5];
			d[2] = dx*transform[6] + dy*transform[7] + dz*transform[8];
			o[0] = origin[0];
			o[1] = origin[1];
			o[2] = origin[2];
			
			ray_depth = 0;
			float attenuation = 1;
			hit = false;
			while (intersect(octree, depth, d, o, n)) {
				ray_depth += 1;
				if (ray_depth >= 5) break;
				float dot = n[0]*s[0] + n[1]*s[1] + n[2]*s[2];
				dot = absf(dot);
				attenuation *= dot;
				reflect_diffuse(o, d, n, &seed);
				hit = true;
			}
			if (hit) {
				c[0] = c[1] = c[2] = attenuation;
			} else {
				c[0] = c[1] = c[2] = 1;
			}
			output[index] = sinv * (output[index] * num_samples + c[0]);
			output[index+1] = sinv * (output[index+1] * num_samples + c[1]);
			output[index+2] = sinv * (output[index+2] * num_samples + c[2]);
			#endif
		}
	}
	
	seeds[ix + iy*nx] = seed;
}
