/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.main;

import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JFrame;
import javax.swing.JPanel;

import se.llbit.chunky.renderer.cl.RenderManager;
import se.llbit.chunky.renderer.ui.Canvas3D;

@SuppressWarnings("serial")
public class TestCL extends JFrame implements MouseListener,
	MouseMotionListener, KeyListener {
	
	private Canvas3D canvas;
	private RenderManager manager;
	
	private int x0 = 0;
	private int y0 = 0;
	
	public TestCL(String[] args) {
		super("OpenCL Test");
		
		JPanel panel = new JPanel(new BorderLayout());
		getContentPane().add(panel);
		
		canvas = new Canvas3D();
		panel.add(canvas, BorderLayout.CENTER);
		
		pack();
		canvas.init();
		
		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		
		try {
			manager = new RenderManager(canvas, args);
			manager.start();
		} catch (Throwable t) {
			t.printStackTrace();
			System.exit(ERROR);
		}
		
		canvas.addMouseMotionListener(this);
		canvas.addMouseListener(this);
		canvas.addKeyListener(this);
	}
	
	@Override
    public void mouseMoved(MouseEvent e) {
    }
    @Override
    public void mouseDragged(MouseEvent e) {
        int dx = e.getX() - x0;
        int dy = e.getY() - y0;
        manager.rotateView(
                - (Math.PI / 250) * dx,
                (Math.PI / 250) * dy);
        x0 = e.getX();
        y0 = e.getY();
    }
    
    @Override
    public void mouseReleased(MouseEvent e) {
    }
    @Override
    public void mousePressed(MouseEvent e) {
        x0 = e.getX();
        y0 = e.getY();
    }
    @Override
    public void mouseExited(MouseEvent e) {
    }
    @Override
    public void mouseEntered(MouseEvent e) {
    }
    @Override
    public void mouseClicked(MouseEvent e) {
    }
    
    @Override
    public void keyTyped(KeyEvent e) {
        onKeyDown(e);
    }
    @Override
    public void keyReleased(KeyEvent e) {
    }
    @Override
    public void keyPressed(KeyEvent e) {
        onKeyDown(e);
    }
    private void onKeyDown(KeyEvent e) {
        switch (e.getKeyCode()) {
        case KeyEvent.VK_A:
        case KeyEvent.VK_LEFT:
        	manager.strafeLeft(1);
            break;
        case KeyEvent.VK_D:
        case KeyEvent.VK_RIGHT:
        	manager.strafeRight(1);
            break;
        case KeyEvent.VK_W:
        case KeyEvent.VK_UP:
        	manager.moveForward(1);
            break;
        case KeyEvent.VK_S:
        case KeyEvent.VK_DOWN:
        	manager.moveBackward(1);
            break;
        case KeyEvent.VK_K:
        	manager.moveForward(100);
            break;
        case KeyEvent.VK_J:
        	manager.moveBackward(100);
            break;
        case KeyEvent.VK_R:
        	manager.moveUp(1);
            break;
        case KeyEvent.VK_F:
        	manager.moveDown(1);
            break;
        }
    }
	
	public static void main(String[] args) {
		new TestCL(args).setVisible(true);
	}

}
