package se.llbit.chunky.renderer;

import java.io.File;
import java.io.IOException;

import se.llbit.chunky.main.Chunky;
import se.llbit.chunky.main.SceneLoadingError;
import se.llbit.chunky.renderer.ui.RenderControls;

public class SceneManager extends Thread {
	
	private final RenderManager controller;
	private final File scenesDir;
	private String sceneName = "";
	private boolean load;
	private RenderControls controls;
	
	public SceneManager(RenderControls controls, RenderManager controller) {
		
		super("Scene Manager");
		
		this.controller = controller;
		this.controls = controls;
		scenesDir = RenderManager.createScenesDirectory();
	}
	
	public void run() {
		try {
			while (!isInterrupted()) {
				synchronized (this) {
					while (sceneName.isEmpty())
						wait();
					String scene = sceneName;
					sceneName = "";
					if (load) {
						try {
							controller.loadScene(controls, new File(scenesDir, scene + ".cvf"));
						} catch (IOException e) {
							Chunky.logger.warning("Could not load scene.\n" +
									"Reason: " + e.getMessage());
						} catch (SceneLoadingError e) {
							Chunky.logger.warning("Could not open scene description.\n" +
									"Reason: " + e.getMessage());
						} catch (InterruptedException e) {
							Chunky.logger.warning("Scene loading was interrupted.");
						}
					} else {
						try {
							controller.saveScene(new File(scenesDir, scene + ".cvf"));
							controls.sceneSaved();
						} catch (IOException e1) {
							Chunky.logger.warning("Could not save scene.\n" +
									"Reason: " + e1.getMessage());
						} catch (InterruptedException e1) {
							Chunky.logger.warning("Scene saving was interrupted.");
						}
					}
				}
			}
		} catch (InterruptedException e) {
		}
	}
	
	public synchronized void loadScene(String name) {
		sceneName = name;
		load = true;
		notify();
	}
	
	public synchronized void saveScene(String name) {
		sceneName = name;
		load = false;
		notify();
	}

}
