/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.renderer.ui;

import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import se.llbit.chunky.renderer.RenderManager;

/**
 * @author Jesper Öqvist (jesper@llbit.se)
 */
@SuppressWarnings("serial")
public class SceneSelector extends JDialog implements ListSelectionListener {

	private DefaultListModel listModel;
	private JList worldList;
	private RenderControls controls;
	protected boolean sceneLoaded = false;

	/**
	 * Creates new form WorldSelector 
	 * @param chunky
	 */
	public SceneSelector(RenderControls controls) {
		this.controls = controls;
		
		initComponents();

		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		setModalityType(Dialog.ModalityType.DOCUMENT_MODAL);
		
		setTitle("Load Scene");
		pack();
		setLocationRelativeTo(controls);
		setVisible(true);
		requestFocus();
	}

	private void initComponents() {

		JLabel listDescription = new javax.swing.JLabel();
		JButton cancelBtn = new JButton("Cancel");

		listModel = new DefaultListModel();
		worldList = new JList(listModel);
		worldList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		worldList.addListSelectionListener(this);
		File[] sceneFiles = RenderManager.createScenesDirectory()
				.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return name.endsWith(".cvf");
			}
		});
		for (File sceneFile : sceneFiles) {
			String name = sceneFile.getName();
			listModel.addElement(name.substring(0, name.length()-4));
		}
		JScrollPane scrollPane = new JScrollPane(worldList);
		
		listDescription.setText("Scene files:");

		cancelBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SceneSelector.this.dispose();
			}
		});

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(
			layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
			.addGroup(layout.createSequentialGroup()
				.addContainerGap()
				.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
					.addComponent(listDescription, javax.swing.GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE)
					.addComponent(scrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE)
					.addComponent(cancelBtn, javax.swing.GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE))
				.addContainerGap())
		);
		layout.setVerticalGroup(
			layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
			.addGroup(layout.createSequentialGroup()
				.addContainerGap()
				.addComponent(listDescription)
				.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
				.addComponent(scrollPane)
				.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
				.addComponent(cancelBtn)
				.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
		);

		pack();
	}

	@Override
	public void valueChanged(ListSelectionEvent event) {
		int selected = event.getFirstIndex();
		String sceneName = (String) listModel.get(selected);
		dispose();
		controls.loadScene(sceneName);
	}
}
