/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.world;

import se.llbit.chunky.model.BedModel;
import se.llbit.chunky.model.BlockModel;
import se.llbit.chunky.model.BookshelfModel;
import se.llbit.chunky.model.ButtonModel;
import se.llbit.chunky.model.CactusModel;
import se.llbit.chunky.model.CakeModel;
import se.llbit.chunky.model.ChestModel;
import se.llbit.chunky.model.CropsModel;
import se.llbit.chunky.model.DoorModel;
import se.llbit.chunky.model.DoubleSlabModel;
import se.llbit.chunky.model.EnchantmentTableModel;
import se.llbit.chunky.model.EndPortalFrameModel;
import se.llbit.chunky.model.EndPortalModel;
import se.llbit.chunky.model.FarmlandModel;
import se.llbit.chunky.model.FenceGateModel;
import se.llbit.chunky.model.FenceModel;
import se.llbit.chunky.model.FurnaceModel;
import se.llbit.chunky.model.GlassPaneModel;
import se.llbit.chunky.model.GrassModel;
import se.llbit.chunky.model.JukeboxModel;
import se.llbit.chunky.model.LadderModel;
import se.llbit.chunky.model.LavaModel;
import se.llbit.chunky.model.LeafModel;
import se.llbit.chunky.model.LeverModel;
import se.llbit.chunky.model.MelonStemModel;
import se.llbit.chunky.model.PancakeModel;
import se.llbit.chunky.model.PistonExtensionModel;
import se.llbit.chunky.model.PistonModel;
import se.llbit.chunky.model.PressurePlateModel;
import se.llbit.chunky.model.PumpkinModel;
import se.llbit.chunky.model.RailModel;
import se.llbit.chunky.model.RedstoneRepeaterModel;
import se.llbit.chunky.model.SaplingModel;
import se.llbit.chunky.model.SignPostModel;
import se.llbit.chunky.model.SlabModel;
import se.llbit.chunky.model.SnowModel;
import se.llbit.chunky.model.SpriteModel;
import se.llbit.chunky.model.StairModel;
import se.llbit.chunky.model.TexturedBlockModel;
import se.llbit.chunky.model.TallGrassModel;
import se.llbit.chunky.model.TorchModel;
import se.llbit.chunky.model.TrapdoorModel;
import se.llbit.chunky.model.VineModel;
import se.llbit.chunky.model.WallSignModel;
import se.llbit.chunky.model.WaterModel;
import se.llbit.chunky.model.WoodModel;
import se.llbit.chunky.model.WorkbenchModel;
import se.llbit.math.Ray;

/**
 * @author Jesper Öqvist <jesper.oqvist@cs.lth.se>
 */
@SuppressWarnings("javadoc")
public enum Block {
	AIR				(0x00, "Air", Texture.air, 1.000293f),
	STONE			(0x01, "Stone", Texture.stone, true),
	GRASS			(0x02, "Grass", Texture.grassTop, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return GrassModel.intersect(ray);
		}
	},
	DIRT			(0x03, "Dirt", Texture.dirt, true),
	COBBLESTONE		(0x04, "Cobblestone", Texture.cobblestone, true),
	WOODENPLANKS	(0x05, "Wooden Planks", Texture.oakPlanks, true),
	SAPLING			(0x06, "Sapling", Texture.oakSapling, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SaplingModel.intersect(ray);
		}
	},
	BEDROCK			(0x07, "Bedrock", Texture.bedrock, true),
	WATER			(0x08, "Water", Texture.water, 1.333f, true) {
		@Override
		public boolean intersect(Ray ray) {
			return WaterModel.intersect(ray);
		}
	},
	STATIONARYWATER	(0x09, "Stationary Water", Texture.water, 1.333f, true) {
		@Override
		public boolean intersect(Ray ray) {
			return WaterModel.intersect(ray);
		}
	},
	LAVA			(0x0A, "Lava", Texture.lava, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return LavaModel.intersect(ray);
		}
	},
	STATIONARYLAVA	(0x0B, "Stationary Lava", Texture.lava, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return LavaModel.intersect(ray);
		}
	},
	SAND			(0x0C, "Sand", Texture.sand, true),
	GRAVEL			(0x0D, "Gravel", Texture.gravel, true),
	GOLDORE			(0x0E, "Gold Ore", Texture.goldOre, true),
	IRONORE			(0x0F, "Iron Ore", Texture.ironOre, true),
	COALORE			(0x10, "Coal Ore", Texture.coalOre, true),
	WOOD			(0x11, "Wood Log", Texture.oakWood, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return WoodModel.intersect(ray);
		}
	},
	LEAVES			(0x12, "Leaves", Texture.oakLeaves, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return LeafModel.intersect(ray);
		}
	},
	SPONGE			(0x13, "Sponge", Texture.sponge, true),
	GLASS			(0x14, "Glass", Texture.glass, 1.520f, false),
	LAPISLAZULIORE	(0x15, "Lapis Lazuli Ore", Texture.lapislazuliOre, true),
	LAPISLAZULIBLOCK(0x16, "Lapis Lazuli Block", Texture.lapislazuliBlock, true),
	DISPENSER		(0x17, "Dispenser", Texture.dispenserFront, true, true) {
		final Texture[] tex = {
				Texture.dispenserFront,
				Texture.furnaceSide,
				Texture.furnaceSide,
				Texture.furnaceSide,
				Texture.furnaceTop,
				Texture.furnaceTop,
		};
		@Override
		public boolean intersect(Ray ray) {
			return FurnaceModel.intersect(ray, tex);
		}
	},
	SANDSTONE		(0x18, "Sandstone", Texture.sandstoneSide, true, true) {
		final Texture[][] tex = {
			// normal
			{
				Texture.sandstoneSide,
				Texture.sandstoneSide,
				Texture.sandstoneSide,
				Texture.sandstoneSide,
				Texture.sandstoneTop,
				Texture.sandstoneBottom,
			},
			
			// decorated
			{
				Texture.sandstoneDecorated,
				Texture.sandstoneDecorated,
				Texture.sandstoneDecorated,
				Texture.sandstoneDecorated,
				Texture.sandstoneTop,
				Texture.sandstoneBottom,
			},
			
			// smooth
			{
				Texture.sandstoneSmooth,
				Texture.sandstoneSmooth,
				Texture.sandstoneSmooth,
				Texture.sandstoneSmooth,
				Texture.sandstoneTop,
				Texture.sandstoneBottom,
			},
		};
		@Override
		public boolean intersect(Ray ray) {
			return TexturedBlockModel.intersect(ray,
					tex[ray.getBlockMetadata() % 3]);
		}
	},
	NOTEBLOCK		(0x19, "Note Block", Icon.noteBlock, Texture.jukeboxSide, true),
	BED				(0x1A, "Bed", Icon.bed, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return BedModel.intersect(ray);
		}
	},
	POWEREDRAIL		(0x1B, "Powered Rail", Texture.poweredRailsOn, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return RailModel.intersect(ray,
					Texture.poweredRails[ray.getBlockMetadata() >>> 3],
					(ray.getBlockMetadata() & 7) % 6);
		}
	},
	DETECTORRAIL	(0x1C, "Detector Rail", Texture.detectorRails, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return RailModel.intersect(ray, Texture.detectorRails,
					ray.getBlockMetadata() % 6);
		}
	},
	STICKYPISTON	(0x1D, "Sticky Piston", Texture.stickyPistonArm, true, true) {
		
		@Override
		public boolean intersect(Ray ray) {
			return PistonModel.intersect(ray, 1);
		}
	},
	COBWEB			(0x1E, "Cobweb", Texture.cobweb, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.cobweb);
		}
	},
	TALLGRASS		(0x1F, "Tall Grass", Texture.tallGrass, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return TallGrassModel.intersect(ray);
		}
	},
	DEADBUSH		(0x20, "Dead Bush", Texture.deadBush, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.deadBush);
		}
	},
	PISTON			(0x21, "Piston", Texture.pistonArm, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return PistonModel.intersect(ray, 0);
		}
	},
	PISTONEXTENSION	(0x22, "Piston Extension", Texture.pistonArm, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return PistonExtensionModel.intersect(ray);
		}
	},
	WOOL			(0x23, "Wool", Texture.lightGrayWool, true),
	MOVEDBYPISTON	(0x24, "Block moved by Piston", Texture.unknown, false),
	YELLOWFLOWER	(0x25, "Yellow Flower", Texture.yellowFlower, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.yellowFlower);
		}
	},
	REDROSE			(0x26, "Red Rose", Texture.redRose, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.redRose);
		}
	},
	BROWNMUSHROOM	(0x27, "Brown Mushroom", Texture.brownMushroom, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.brownMushroom);
		}
	},
	REDMUSHROOM		(0x28, "Red Mushroom", Texture.redMushroom, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.redMushroom);
		}
	},
	GOLDBLOCK		(0x29, "Gold Block", Texture.goldBlock, true),
	IRONBLOCK		(0x2A, "Iron Block", Texture.ironBlock, true),
	DOUBLESLAB		(0x2B, "Double Stone Slab", Texture.slabTop, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			switch (ray.getBlockMetadata() & 7) {
			case 1:
				return DoubleSlabModel.intersect(ray, Texture.sandstoneSide, Texture.sandstoneTop);
			case 2:
				return DoubleSlabModel.intersect(ray, Texture.oakPlanks, Texture.oakPlanks);
			case 3:
				return DoubleSlabModel.intersect(ray, Texture.cobblestone, Texture.cobblestone);
			case 4:
				return DoubleSlabModel.intersect(ray, Texture.brick, Texture.brick);
			case 5:
				return DoubleSlabModel.intersect(ray, Texture.stoneBrick, Texture.stoneBrick);
			case 0:
			default:
				return DoubleSlabModel.intersect(ray, Texture.slabSide, Texture.slabTop);
			}
		}
	},
	SLAB			(0x2C, "Stone Slab", Texture.slabTop, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			switch (ray.getBlockMetadata() & 7) {
			case 1:
				return SlabModel.intersect(ray, Texture.sandstoneSide, Texture.sandstoneTop);
			case 2:
				return SlabModel.intersect(ray, Texture.oakPlanks, Texture.oakPlanks);
			case 3:
				return SlabModel.intersect(ray, Texture.cobblestone, Texture.cobblestone);
			case 4:
				return SlabModel.intersect(ray, Texture.brick, Texture.brick);
			case 5:
				return SlabModel.intersect(ray, Texture.stoneBrick, Texture.stoneBrick);
			case 0:
			default:
				return SlabModel.intersect(ray, Texture.slabSide, Texture.slabTop);
			}
		}
	},
	BRICKS			(0x2D, "Bricks", Texture.brick, true),
	TNT				(0x2E, "TNT", Texture.tntSide, true, true) {
		final Texture[] tex = {
			Texture.tntSide,
			Texture.tntSide,
			Texture.tntSide,
			Texture.tntSide,
			Texture.tntTop,
			Texture.tntBottom,
		};
		@Override
		public boolean intersect(Ray ray) {
			return TexturedBlockModel.intersect(ray, tex);
		}
	},
	BOOKSHELF		(0x2F, "Bookshelf", Texture.bookshelf, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return BookshelfModel.intersect(ray);
		}
	},
	MOSSSTONE		(0x30, "Moss Stone", Texture.mossStone, true),
	OBSIDIAN		(0x31, "Obsidian", Texture.obsidian, true),
	TORCH			(0x32, "Torch", Texture.torch, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return TorchModel.intersect(ray, Texture.torch);
		}
	},
	FIRE			(0x33, "Fire", Texture.fire, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.fire);
		}
	},
	MONSTERSPAWNER	(0x34, "Monster Spawner", Texture.monsterSpawner, false),
	WOODENSTAIRS	(0x35, "Wooden Stairs", Icon.woodenStairs, Texture.oakPlanks, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return StairModel.intersect(ray, Texture.oakPlanks);
		}
	},
	CHEST			(0x36, "Chest", Texture.chestFront, true, true) {
		final Texture[][] tex = {
				// single
				{
					Texture.chestFront,
					Texture.chestBack,
					Texture.chestLeft,
					Texture.chestRight,
					Texture.chestTop,
					Texture.chestBottom,
					Texture.chestLock,
					Texture.chestLock,
					Texture.chestLock,
					Texture.chestLock,
					Texture.chestLock,
				},
				
				// left
				{
					Texture.largeChestFrontLeft,
					Texture.largeChestBackLeft,
					Texture.largeChestLeft,
					Texture.largeChestTopLeft,
					Texture.largeChestBottomLeft,
					Texture.chestLock,
					Texture.chestLock,
					Texture.chestLock,
					Texture.chestLock,
				},
				
				// right
				{
					Texture.largeChestFrontRight,
					Texture.largeChestBackRight,
					Texture.largeChestRight,
					Texture.largeChestTopRight,
					Texture.largeChestBottomRight,
					Texture.chestLock,
					Texture.chestLock,
					Texture.chestLock,
					Texture.chestLock,
				}
		};
		@Override
		public boolean intersect(Ray ray) {
			return ChestModel.intersect(ray, tex[(ray.currentMaterial >> 16) % 3]);
		}
	},
	REDSTONEWIRE	(0x37, "Redstone Wire", Texture.redstoneWireCross, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return PancakeModel.intersect(ray, Texture.redstoneWireCross);
		}
	},
	DIAMONDORE		(0x38, "Diamond Ore", Texture.diamondOre, true),
	DIAMONDBLOCK	(0x39, "Diamond Block", Texture.diamondBlock, true),
	WORKBENCH		(0x3A, "Workbench", Texture.workbenchFront, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return WorkbenchModel.intersect(ray);
		}
	},
	CROPS			(0x3B, "Crops", Texture.wheat8, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return CropsModel.intersect(ray,
					Texture.wheat[ray.getBlockMetadata() % 8]);
		}
	},
	SOIL			(0x3C, "Soil", Texture.farmlandWet, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return FarmlandModel.intersect(ray);
		}
	},
	FURNACEUNLIT	(0x3D, "Furnace", Texture.furnaceUnlitFront, true, true) {
		final Texture[] tex = {
				Texture.furnaceUnlitFront,
				Texture.furnaceSide,
				Texture.furnaceSide,
				Texture.furnaceSide,
				Texture.furnaceTop,
				Texture.furnaceTop,
		};
		@Override
		public boolean intersect(Ray ray) {
			return FurnaceModel.intersect(ray, tex);
		}
	},
	FURNACELIT		(0x3E, "Burning Furnace", Texture.furnaceLitFront, true, true) {
		final Texture[] tex = {
				Texture.furnaceLitFront,
				Texture.furnaceSide,
				Texture.furnaceSide,
				Texture.furnaceSide,
				Texture.furnaceTop,
				Texture.furnaceTop,
		};
		@Override
		public boolean intersect(Ray ray) {
			return FurnaceModel.intersect(ray, tex);
		}
	},
	SIGNPOST		(0x3F, "Sign Post", Icon.signPost, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SignPostModel.intersect(ray);
		}
	},
	WOODENDOOR		(0x40, "Wooden Door", Icon.woodenDoor, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return DoorModel.intersect(ray,
					Texture.woodenDoor[ray.getBlockMetadata() >>> 3]);
		}
	},
	LADDER			(0x41, "Ladder", Texture.ladder, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return LadderModel.intersect(ray);
		}
	},
	MINECARTTRACKS	(0x42, "Minecart Tracks", Texture.rails, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			int type = ray.getBlockMetadata() % 10;
			return RailModel.intersect(ray, Texture.railsType[type], type);
		}
	},
	STONESTAIRS		(0x43, "Stone Stairs", Icon.stoneStairs, Texture.stone, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return StairModel.intersect(ray, Texture.stone);
		}
	},
	WALLSIGN		(0x44, "Wall Sign", Icon.wallSign, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return WallSignModel.intersect(ray);
		}
	},
	LEVER			(0x45, "Lever", Texture.lever, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return LeverModel.intersect(ray);
		}
	},
	STONEPRESSUREPLATE(0x46, "Stone Pressure Plate", Icon.stonePressurePlate, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return PressurePlateModel.intersect(ray, Texture.stone);
		}
	},
	IRONDOOR		(0x47, "Iron Door", Icon.ironDoor, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return DoorModel.intersect(ray,
					Texture.ironDoor[ray.getBlockMetadata() >>> 3]);
		}
	},
	WOODENPRESSUREPLATE(0x48, "Wooden Pressure Plate", Icon.woodenPressurePlate, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return PressurePlateModel.intersect(ray, Texture.oakPlanks);
		}
	},
	REDSTONEORE		(0x49, "Redstone Ore", Texture.redstoneOre, true),
	GLOWINGREDSTONEORE(0x4A, "Glowing Redstone Ore", Texture.redstoneOre, true),
	REDSTONETORCHOFF(0x4B, "Redstone Torch (off)", Texture.redstoneTorchOff, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return TorchModel.intersect(ray, Texture.redstoneTorchOff);
		}
	},
	REDSTONETORCHON	(0x4C, "Redstone Torch (on)", Texture.redstoneTorchOn, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return TorchModel.intersect(ray, Texture.redstoneTorchOn);
		}
	},
	STONEBUTTON		(0x4D, "Stone Button", Icon.stoneButton, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return ButtonModel.intersect(ray);
		}
	},
	SNOW			(0x4E, "Snow", Texture.snowBlock, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SnowModel.intersect(ray);
		}
	},
	ICE				(0x4F, "Ice", Texture.ice, 1.31f),
	SNOWBLOCK		(0x50, "Snow Block", Texture.snowBlock, true),
	CACTUS			(0x51, "Cactus", Texture.cactusSide, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return CactusModel.intersect(ray);
		}
	},
	CLAY			(0x52, "Clay", Texture.clay, true),
	SUGARCANE		(0x53, "Sugar Cane", Texture.sugarCane, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.sugarCane);
		}
	},
	JUKEBOX			(0x54, "Jukebox", Texture.jukeboxSide, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return JukeboxModel.intersect(ray);
		}
	},
	FENCE			(0x55, "Fence", Icon.fence, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return FenceModel.intersect(ray, Texture.oakPlanks);
		}
	},
	PUMPKIN			(0x56, "Pumpkin", Texture.pumpkinSide, true, true) {
		final Texture[] tex = {
				Texture.pumpkinFront,
				Texture.pumpkinSide,
				Texture.pumpkinSide,
				Texture.pumpkinSide,
				Texture.pumpkinTop,
				Texture.pumpkinTop,
		};
		@Override
		public boolean intersect(Ray ray) {
			return PumpkinModel.intersect(ray, tex);
		}
	},
	NETHERRACK		(0x57, "Netherrack", Texture.netherrack, true),
	SOULSAND		(0x58, "Soul Sand", Texture.soulsand, true),
	GLOWSTONE		(0x59, "Glowstone", Texture.glowstone, true),
	PORTAL			(0x5A, "Portal", Texture.portal, false),
	JACKOLANTERN	(0x5B, "Jack-O-Lantern", Texture.jackolanternFront, true, true) {
		final Texture[] tex = {
				Texture.jackolanternFront,
				Texture.pumpkinSide,
				Texture.pumpkinSide,
				Texture.pumpkinSide,
				Texture.pumpkinTop,
				Texture.pumpkinTop,
		};
		@Override
		public boolean intersect(Ray ray) {
			return PumpkinModel.intersect(ray, tex);
		}
	},
	CAKE			(0x5C, "Cake Block", Icon.cake, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return CakeModel.intersect(ray);
		}
	},
	REDSTONEREPEATEROFF(0x5D, "Redstone Repeater (off)", Texture.redstoneRepeaterOff, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return RedstoneRepeaterModel.intersect(ray, Texture.redstoneRepeaterOff);
		}
	},
	REDSTONEREPEATERON(0x5E, "Redstone Repeater (on)", Texture.redstoneRepeaterOn, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return RedstoneRepeaterModel.intersect(ray, Texture.redstoneRepeaterOn);
		}
	},
	LOCKEDCHEST		(0x5F, "Locked Chest", Texture.chestFront, true, true) {
		final Texture[] tex = {
				Texture.chestFront,
				Texture.chestBack,
				Texture.chestLeft,
				Texture.chestRight,
				Texture.chestTop,
				Texture.chestBottom,
				Texture.chestLock,
				Texture.chestLock,
				Texture.chestLock,
				Texture.chestLock,
				Texture.chestLock,
		};
		@Override
		public boolean intersect(Ray ray) {
			return ChestModel.intersect(ray, tex);
		}
	},
	TRAPDOOR		(0x60, "Trapdoor", Texture.trapdoor, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return TrapdoorModel.intersect(ray);
		}
	},
	HIDDENSILVERFISH(0x61, "Hidden Silverfish", Texture.stone, true),
	STONEBRICKS		(0x62, "Stone Bricks", Texture.stoneBrick, true, true) {
		@Override
		public boolean intersect(Ray ray) {
			return BlockModel.intersect(ray, Texture.stoneBrickType[(ray.currentMaterial>>8)&3]);
		}
	},
	HUGEBROWNMUSHROOM(0x63, "Huge Brown Mushroom", Texture.hugeBrownMushroom, true, true) {
		final Texture[][] tex =  {
				// fleshy
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores
				},
				
				// cap on top, west, north
				{
					Texture.hugeBrownMushroom, Texture.mushroomPores,
					Texture.mushroomPores, Texture.hugeBrownMushroom,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top, north
				{
					Texture.hugeBrownMushroom, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top, east, north
				{
					Texture.hugeBrownMushroom, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top, west
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.hugeBrownMushroom,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top, east
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top, west, south
				{
					Texture.mushroomPores, Texture.hugeBrownMushroom,
					Texture.mushroomPores, Texture.hugeBrownMushroom,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top, south
				{
					Texture.mushroomPores, Texture.hugeBrownMushroom,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// cap on top, east, south
				{
					Texture.mushroomPores, Texture.hugeBrownMushroom,
					Texture.hugeBrownMushroom, Texture.mushroomPores,
					Texture.hugeBrownMushroom, Texture.mushroomPores
				},
				
				// stem
				{
					Texture.mushroomStem, Texture.mushroomStem,
					Texture.mushroomStem, Texture.mushroomStem,
					Texture.mushroomPores, Texture.mushroomPores
				},
		};
		@Override
		public boolean intersect(Ray ray) {
			return TexturedBlockModel.intersect(ray, tex[ray.getBlockMetadata() % 11]);
		}
	},
	HUGEREDMUSHROOM	(0x64, "Huge Red Mushroom", Texture.hugeRedMushroom, true, true) {
		final Texture[][] tex =  {
				// fleshy
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores
				},
				
				// cap on top, west, north
				{
					Texture.hugeRedMushroom, Texture.mushroomPores,
					Texture.mushroomPores, Texture.hugeRedMushroom,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top, north
				{
					Texture.hugeRedMushroom, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top, east, north
				{
					Texture.hugeRedMushroom, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top, west
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.hugeRedMushroom,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top, east
				{
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top, west, south
				{
					Texture.mushroomPores, Texture.hugeRedMushroom,
					Texture.mushroomPores, Texture.hugeRedMushroom,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top, south
				{
					Texture.mushroomPores, Texture.hugeRedMushroom,
					Texture.mushroomPores, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// cap on top, east, south
				{
					Texture.mushroomPores, Texture.hugeRedMushroom,
					Texture.hugeRedMushroom, Texture.mushroomPores,
					Texture.hugeRedMushroom, Texture.mushroomPores
				},
				
				// stem
				{
					Texture.mushroomStem, Texture.mushroomStem,
					Texture.mushroomStem, Texture.mushroomStem,
					Texture.mushroomPores, Texture.mushroomPores
				},
		};
		@Override
		public boolean intersect(Ray ray) {
			return TexturedBlockModel.intersect(ray, tex[ray.getBlockMetadata() % 11]);
		}
	},
	IRONBARS		(0x65, "Iron Bars", Texture.ironBars, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return GlassPaneModel.intersect(ray, Texture.ironBars, Texture.ironBars);
		}
	},
	GLASSPANE		(0x66, "Glass Pane", Texture.glass, 1.520f, true) {
		@Override
		public boolean intersect(Ray ray) {
			return GlassPaneModel.intersect(ray, Texture.glass, Texture.glassPaneSide);
		}
	},
	MELON			(0x67, "Melon", Texture.melonSide, true, true) {
		final Texture tex[] = {
				Texture.melonSide, Texture.melonSide,
				Texture.melonSide, Texture.melonSide,
				Texture.melonTop, Texture.melonTop
		};
		
		@Override
		public boolean intersect(Ray ray) {
			return TexturedBlockModel.intersect(ray, tex);
		}
	},
	PUMPKINSTEM		(0x68, "Pumpkin Stem", Texture.melonStem, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return MelonStemModel.intersect(ray);
		}
	},
	MELONSTEM		(0x69, "Melon Stem", Texture.melonStem, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return MelonStemModel.intersect(ray);
		}
	},
	VINES			(0x6A, "Vines", Texture.vines, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return VineModel.intersect(ray);
		}
	},
	FENCEGATE		(0x6B, "Fence Gate", Texture.unknown, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return FenceGateModel.intersect(ray);
		}
	},
	BRICKSTAIRS		(0x6C, "Brick Stairs", Texture.unknown, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return StairModel.intersect(ray, Texture.brick);
		}
	},
	STONEBRICKSTAIRS(0x6D, "Stone Brick Stairs", Icon.stoneBrickStairs, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return StairModel.intersect(ray, Texture.stoneBrick);
		}
	},
	MYCELIUM		(0x6E, "Mycelium", Texture.myceliumSide, Texture.myceliumTop, true, true) {
		final Texture tex[] = {
				Texture.myceliumSide, Texture.myceliumSide,
				Texture.myceliumSide, Texture.myceliumSide,
				Texture.myceliumTop, Texture.dirt
		};
		
		@Override
		public boolean intersect(Ray ray) {
			return TexturedBlockModel.intersect(ray, tex);
		}
	},
	LILYPAD			(0x6F, "Lily Pad", Texture.lilyPad, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return PancakeModel.intersect(ray, Texture.lilyPad);
		}
	},
	NETHERBRICK		(0x70, "Nether Brick", Texture.netherBrick, true),
	NETHERBRICKFENCE(0x71, "Nether Brick Fence", Texture.unknown, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return FenceModel.intersect(ray, Texture.netherBrick);
		}
	},
	NETHERBRICKSTAIRS(0x72, "Nether Brick Stairs", Texture.unknown, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return StairModel.intersect(ray, Texture.netherBrick);
		}
	},
	NETHERWART		(0x73, "Nether Wart", Texture.netherWart3, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return CropsModel.intersect(ray,
					Texture.netherWart[ray.getBlockMetadata() & 3]);
		}
	},
	ENCHNATMENTTABLE(0x74, "Enchantment Table", Texture.enchantmentTableSide, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return EnchantmentTableModel.intersect(ray);
		}
	},
	BREWINGSTAND	(0x75, "Brewing Stand", Texture.brewingStandSide, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return SpriteModel.intersect(ray, Texture.brewingStandSide);
		}
	},
	CAULDRON		(0x76, "Cauldron", Texture.cauldronSide, false),
	ENDPORTAL		(0x77, "End Portal", Texture.endPortal, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return EndPortalModel.intersect(ray);
		}
	},
	ENDPORTALFRAME	(0x78, "End Portal Frame", Texture.endPortalFrameTop, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return EndPortalFrameModel.intersect(ray);
		}
	},
	ENDSTONE		(0x79, "End Stone", Texture.endStone, true),
	DRAGONEGG		(0x7A, "Dragon Egg", Texture.dragonEgg, false),
	REDSTONELAMPOFF	(0x7B, "Redstone Lamp (off)", Texture.redstoneLampOff, true),
	REDSTONELAMPON	(0x7C, "Redstone Lamp (on)", Texture.redstoneLampOn, true),
	DOUBLEWOODENSLAB(0x7D, "Double Wooden Slab", Texture.oakPlanks, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			switch (ray.getBlockMetadata() & 7) {
			case 1:
				return SlabModel.intersect(ray, Texture.pinePlanks, Texture.pinePlanks);
			case 2:
				return SlabModel.intersect(ray, Texture.birchPlanks, Texture.birchPlanks);
			case 3:
				return SlabModel.intersect(ray, Texture.jungleTreePlanks, Texture.jungleTreePlanks);
			case 0:
			default:
				return SlabModel.intersect(ray, Texture.oakPlanks, Texture.oakPlanks);
			}
		}
	},
	SINGLEWOODENSLAB(0x7E, "Single Wooden Slab", Texture.oakPlanks, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			switch (ray.getBlockMetadata() & 7) {
			case 1:
				return SlabModel.intersect(ray, Texture.pinePlanks, Texture.pinePlanks);
			case 2:
				return SlabModel.intersect(ray, Texture.birchPlanks, Texture.birchPlanks);
			case 3:
				return SlabModel.intersect(ray, Texture.jungleTreePlanks, Texture.jungleTreePlanks);
			case 0:
			default:
				return SlabModel.intersect(ray, Texture.oakPlanks, Texture.oakPlanks);
			}
		}
	},
	COCOAPLANT		(0x7F, "Cocoa Plant", Texture.cocoaPlant3, false),
	SANDSTONESTAIRS	(0x80, "Sandstone Stairs", Texture.sandstoneSide, false, true) {
		@Override
		public boolean intersect(Ray ray) {
			return StairModel.intersect(ray, Texture.sandstoneSide,
					Texture.sandstoneTop, Texture.sandstoneBottom);
		}
	},
	EMERALDORE		(0x81, "Emerald Ore", Texture.emeraldOre, true),
	ENDERCHEST		(0x82, "Ender Chest", Texture.unknown, false, true) {
		final Texture[] tex = {
				Texture.enderChestFront,
				Texture.enderChestBack,
				Texture.enderChestLeft,
				Texture.enderChestRight,
				Texture.enderChestTop,
				Texture.enderChestBottom,
				Texture.enderChestLock,
				Texture.enderChestLock,
				Texture.enderChestLock,
				Texture.enderChestLock,
				Texture.enderChestLock,
		};
		@Override
		public boolean intersect(Ray ray) {
			return ChestModel.intersect(ray, tex);
		}
	},
	TRIPWIREHOOK	(0x83, "Tripwire Hook", Texture.tripwireHook, false),
	TRIPWIRE		(0x84, "Tripwire", Texture.tripwire, false),
	EMERALDBLOCK	(0x85, "Emerald Block", Texture.emeraldBlock, true),
	UNKNOWN0x86		(0x86, "Unknown Block 0x86", Texture.unknown, false),
	UNKNOWN0x87		(0x87, "Unknown Block 0x87", Texture.unknown, false),
	UNKNOWN0x88		(0x88, "Unknown Block 0x88", Texture.unknown, false),
	UNKNOWN0x89		(0x89, "Unknown Block 0x89", Texture.unknown, false),
	UNKNOWN0x8A		(0x8A, "Unknown Block 0x8A", Texture.unknown, false),
	UNKNOWN0x8B		(0x8B, "Unknown Block 0x8B", Texture.unknown, false),
	UNKNOWN0x8C		(0x8C, "Unknown Block 0x8C", Texture.unknown, false),
	UNKNOWN0x8D		(0x8D, "Unknown Block 0x8D", Texture.unknown, false),
	UNKNOWN0x8E		(0x8E, "Unknown Block 0x8E", Texture.unknown, false),
	UNKNOWN0x8F		(0x8F, "Unknown Block 0x8F", Texture.unknown, false),
	UNKNOWN0x90		(0x90, "Unknown Block 0x90", Texture.unknown, false),
	UNKNOWN0x91		(0x91, "Unknown Block 0x91", Texture.unknown, false),
	UNKNOWN0x92		(0x92, "Unknown Block 0x92", Texture.unknown, false),
	UNKNOWN0x93		(0x93, "Unknown Block 0x93", Texture.unknown, false),
	UNKNOWN0x94		(0x94, "Unknown Block 0x94", Texture.unknown, false),
	UNKNOWN0x95		(0x95, "Unknown Block 0x95", Texture.unknown, false),
	UNKNOWN0x96		(0x96, "Unknown Block 0x96", Texture.unknown, false),
	UNKNOWN0x97		(0x97, "Unknown Block 0x97", Texture.unknown, false),
	UNKNOWN0x98		(0x98, "Unknown Block 0x98", Texture.unknown, false),
	UNKNOWN0x99		(0x99, "Unknown Block 0x99", Texture.unknown, false),
	UNKNOWN0x9A		(0x9A, "Unknown Block 0x9A", Texture.unknown, false),
	UNKNOWN0x9B		(0x9B, "Unknown Block 0x9B", Texture.unknown, false),
	UNKNOWN0x9C		(0x9C, "Unknown Block 0x9C", Texture.unknown, false),
	UNKNOWN0x9D		(0x9D, "Unknown Block 0x9D", Texture.unknown, false),
	UNKNOWN0x9E		(0x9E, "Unknown Block 0x9E", Texture.unknown, false),
	UNKNOWN0x9F		(0x9F, "Unknown Block 0x9F", Texture.unknown, false),
	UNKNOWN0xA0		(0xA0, "Unknown Block 0xA0", Texture.unknown, false),
	UNKNOWN0xA1		(0xA1, "Unknown Block 0xA1", Texture.unknown, false),
	UNKNOWN0xA2		(0xA2, "Unknown Block 0xA2", Texture.unknown, false),
	UNKNOWN0xA3		(0xA3, "Unknown Block 0xA3", Texture.unknown, false),
	UNKNOWN0xA4		(0xA4, "Unknown Block 0xA4", Texture.unknown, false),
	UNKNOWN0xA5		(0xA5, "Unknown Block 0xA5", Texture.unknown, false),
	UNKNOWN0xA6		(0xA6, "Unknown Block 0xA6", Texture.unknown, false),
	UNKNOWN0xA7		(0xA7, "Unknown Block 0xA7", Texture.unknown, false),
	UNKNOWN0xA8		(0xA8, "Unknown Block 0xA8", Texture.unknown, false),
	UNKNOWN0xA9		(0xA9, "Unknown Block 0xA9", Texture.unknown, false),
	UNKNOWN0xAA		(0xAA, "Unknown Block 0xAA", Texture.unknown, false),
	UNKNOWN0xAB		(0xAB, "Unknown Block 0xAB", Texture.unknown, false),
	UNKNOWN0xAC		(0xAC, "Unknown Block 0xAC", Texture.unknown, false),
	UNKNOWN0xAD		(0xAD, "Unknown Block 0xAD", Texture.unknown, false),
	UNKNOWN0xAE		(0xAE, "Unknown Block 0xAE", Texture.unknown, false),
	UNKNOWN0xAF		(0xAF, "Unknown Block 0xAF", Texture.unknown, false),
	UNKNOWN0xB0		(0xB0, "Unknown Block 0xB0", Texture.unknown, false),
	UNKNOWN0xB1		(0xB1, "Unknown Block 0xB1", Texture.unknown, false),
	UNKNOWN0xB2		(0xB2, "Unknown Block 0xB2", Texture.unknown, false),
	UNKNOWN0xB3		(0xB3, "Unknown Block 0xB3", Texture.unknown, false),
	UNKNOWN0xB4		(0xB4, "Unknown Block 0xB4", Texture.unknown, false),
	UNKNOWN0xB5		(0xB5, "Unknown Block 0xB5", Texture.unknown, false),
	UNKNOWN0xB6		(0xB6, "Unknown Block 0xB6", Texture.unknown, false),
	UNKNOWN0xB7		(0xB7, "Unknown Block 0xB7", Texture.unknown, false),
	UNKNOWN0xB8		(0xB8, "Unknown Block 0xB8", Texture.unknown, false),
	UNKNOWN0xB9		(0xB9, "Unknown Block 0xB9", Texture.unknown, false),
	UNKNOWN0xBA		(0xBA, "Unknown Block 0xBA", Texture.unknown, false),
	UNKNOWN0xBB		(0xBB, "Unknown Block 0xBB", Texture.unknown, false),
	UNKNOWN0xBC		(0xBC, "Unknown Block 0xBC", Texture.unknown, false),
	UNKNOWN0xBD		(0xBD, "Unknown Block 0xBD", Texture.unknown, false),
	UNKNOWN0xBE		(0xBE, "Unknown Block 0xBE", Texture.unknown, false),
	UNKNOWN0xBF		(0xBF, "Unknown Block 0xBF", Texture.unknown, false),
	UNKNOWN0xC0		(0xC0, "Unknown Block 0xC0", Texture.unknown, false),
	UNKNOWN0xC1		(0xC1, "Unknown Block 0xC1", Texture.unknown, false),
	UNKNOWN0xC2		(0xC2, "Unknown Block 0xC2", Texture.unknown, false),
	UNKNOWN0xC3		(0xC3, "Unknown Block 0xC3", Texture.unknown, false),
	UNKNOWN0xC4		(0xC4, "Unknown Block 0xC4", Texture.unknown, false),
	UNKNOWN0xC5		(0xC5, "Unknown Block 0xC5", Texture.unknown, false),
	UNKNOWN0xC6		(0xC6, "Unknown Block 0xC6", Texture.unknown, false),
	UNKNOWN0xC7		(0xC7, "Unknown Block 0xC7", Texture.unknown, false),
	UNKNOWN0xC8		(0xC8, "Unknown Block 0xC8", Texture.unknown, false),
	UNKNOWN0xC9		(0xC9, "Unknown Block 0xC9", Texture.unknown, false),
	UNKNOWN0xCA		(0xCA, "Unknown Block 0xCA", Texture.unknown, false),
	UNKNOWN0xCB		(0xCB, "Unknown Block 0xCB", Texture.unknown, false),
	UNKNOWN0xCC		(0xCC, "Unknown Block 0xCC", Texture.unknown, false),
	UNKNOWN0xCD		(0xCD, "Unknown Block 0xCD", Texture.unknown, false),
	UNKNOWN0xCE		(0xCE, "Unknown Block 0xCE", Texture.unknown, false),
	UNKNOWN0xCF		(0xCF, "Unknown Block 0xCF", Texture.unknown, false),
	UNKNOWN0xD0		(0xD0, "Unknown Block 0xD0", Texture.unknown, false),
	UNKNOWN0xD1		(0xD1, "Unknown Block 0xD1", Texture.unknown, false),
	UNKNOWN0xD2		(0xD2, "Unknown Block 0xD2", Texture.unknown, false),
	UNKNOWN0xD3		(0xD3, "Unknown Block 0xD3", Texture.unknown, false),
	UNKNOWN0xD4		(0xD4, "Unknown Block 0xD4", Texture.unknown, false),
	UNKNOWN0xD5		(0xD5, "Unknown Block 0xD5", Texture.unknown, false),
	UNKNOWN0xD6		(0xD6, "Unknown Block 0xD6", Texture.unknown, false),
	UNKNOWN0xD7		(0xD7, "Unknown Block 0xD7", Texture.unknown, false),
	UNKNOWN0xD8		(0xD8, "Unknown Block 0xD8", Texture.unknown, false),
	UNKNOWN0xD9		(0xD9, "Unknown Block 0xD9", Texture.unknown, false),
	UNKNOWN0xDA		(0xDA, "Unknown Block 0xDA", Texture.unknown, false),
	UNKNOWN0xDB		(0xDB, "Unknown Block 0xDB", Texture.unknown, false),
	UNKNOWN0xDC		(0xDC, "Unknown Block 0xDC", Texture.unknown, false),
	UNKNOWN0xDD		(0xDD, "Unknown Block 0xDD", Texture.unknown, false),
	UNKNOWN0xDE		(0xDE, "Unknown Block 0xDE", Texture.unknown, false),
	UNKNOWN0xDF		(0xDF, "Unknown Block 0xDF", Texture.unknown, false),
	UNKNOWN0xE0		(0xE0, "Unknown Block 0xE0", Texture.unknown, false),
	UNKNOWN0xE1		(0xE1, "Unknown Block 0xE1", Texture.unknown, false),
	UNKNOWN0xE2		(0xE2, "Unknown Block 0xE2", Texture.unknown, false),
	UNKNOWN0xE3		(0xE3, "Unknown Block 0xE3", Texture.unknown, false),
	UNKNOWN0xE4		(0xE4, "Unknown Block 0xE4", Texture.unknown, false),
	UNKNOWN0xE5		(0xE5, "Unknown Block 0xE5", Texture.unknown, false),
	UNKNOWN0xE6		(0xE6, "Unknown Block 0xE6", Texture.unknown, false),
	UNKNOWN0xE7		(0xE7, "Unknown Block 0xE7", Texture.unknown, false),
	UNKNOWN0xE8		(0xE8, "Unknown Block 0xE8", Texture.unknown, false),
	UNKNOWN0xE9		(0xE9, "Unknown Block 0xE9", Texture.unknown, false),
	UNKNOWN0xEA		(0xEA, "Unknown Block 0xEA", Texture.unknown, false),
	UNKNOWN0xEB		(0xEB, "Unknown Block 0xEB", Texture.unknown, false),
	UNKNOWN0xEC		(0xEC, "Unknown Block 0xEC", Texture.unknown, false),
	UNKNOWN0xED		(0xED, "Unknown Block 0xED", Texture.unknown, false),
	UNKNOWN0xEE		(0xEE, "Unknown Block 0xEE", Texture.unknown, false),
	UNKNOWN0xEF		(0xEF, "Unknown Block 0xEF", Texture.unknown, false),
	UNKNOWN0xF0		(0xF0, "Unknown Block 0xF0", Texture.unknown, false),
	UNKNOWN0xF1		(0xF1, "Unknown Block 0xF1", Texture.unknown, false),
	UNKNOWN0xF2		(0xF2, "Unknown Block 0xF2", Texture.unknown, false),
	UNKNOWN0xF3		(0xF3, "Unknown Block 0xF3", Texture.unknown, false),
	UNKNOWN0xF4		(0xF4, "Unknown Block 0xF4", Texture.unknown, false),
	UNKNOWN0xF5		(0xF5, "Unknown Block 0xF5", Texture.unknown, false),
	UNKNOWN0xF6		(0xF6, "Unknown Block 0xF6", Texture.unknown, false),
	UNKNOWN0xF7		(0xF7, "Unknown Block 0xF7", Texture.unknown, false),
	UNKNOWN0xF8		(0xF8, "Unknown Block 0xF8", Texture.unknown, false),
	UNKNOWN0xF9		(0xF9, "Unknown Block 0xF9", Texture.unknown, false),
	UNKNOWN0xFA		(0xFA, "Unknown Block 0xFA", Texture.unknown, false),
	UNKNOWN0xFB		(0xFB, "Unknown Block 0xFB", Texture.unknown, false),
	UNKNOWN0xFC		(0xFC, "Unknown Block 0xFC", Texture.unknown, false),
	UNKNOWN0xFD		(0xFD, "Unknown Block 0xFD", Texture.unknown, false),
	UNKNOWN0xFE		(0xFE, "Unknown Block 0xFE", Texture.unknown, false),
	UNKNOWN0xFF		(0xFF, "Unknown Block 0xFF", Texture.unknown, false);

	public final int id;
	public final float refractionIndex;
	public final boolean localIntersect;
	private final String name;
	private final boolean solid;
	private Texture frontTexture;
	private Texture icon;

	Block(int id, String name, Texture icon, float refractionIndex) {
		this(id, name, icon, icon, false, refractionIndex, false);
	}
	
	Block(int id, String name, Texture icon, float refractionIndex,
			boolean localIntersect) {
		this(id, name, icon, icon, false, refractionIndex, localIntersect);
	}
	
	Block(int id, String name, Texture icon, boolean solid) {
		this(id, name, icon, icon, solid);
	}

	Block(int id, String name, Texture icon, boolean solid,
			boolean localIntersect) {
		this(id, name, icon, icon, solid, 1.000293f, localIntersect);
	}

	Block(int id, String name, Texture icon, Texture texture, boolean solid) {
		this(id, name, icon, texture, solid, 1.000293f, false);
	}
	
	Block(int id, String name, Texture icon, Texture texture, boolean solid,
			boolean localIntersect) {
		this(id, name, icon, texture, solid, 1.000293f, localIntersect);
	}
	
	Block(int id, String name, Texture icon, Texture texture,
			boolean solid, float refractionIndex, boolean localIntersect) {

		this.id = id;
		this.name = name;
		this.solid = solid;
		this.localIntersect = localIntersect;
		this.refractionIndex = refractionIndex;
		this.icon = icon;
		
		setTexture(texture);
	}

	/**
	 * A block is solid if it occupies an entire one meter cube of space
	 * and no light can pass through it.
	 * 
	 * @return {@code true} if the block is solid
	 */
	public boolean isSolid() {
		return solid;
	}
	
	/**
	 * The name of the block.
	 * 
	 * @return
	 */
	public String getBlockName() {
		return name;
	}
	
	/**
	 * Get the average color for the block texture as an
	 * RGB value.
	 * 
	 * @return average color (RGB)
	 */
	public int getAvgRGB() {
		return frontTexture.getAvgColor();
	}
	
	/**
	 * Get the average color for the block texture as an
	 * RGB value.
	 * 
	 * @return average color (RGB)
	 */
	public int getAvgTopRGB() {
		return icon.getAvgColor();
	}
	
	@Override
	public String toString() {
		return getBlockName();
	}

	public static final Block[] values = values();

	public void setTexture(Texture newTexture) {
		frontTexture = newTexture;
	}
	
	public boolean intersect(Ray ray) {
		return BlockModel.intersect(ray, frontTexture);
	}

	public Texture getTexture() {
		return frontTexture;
	}
}
